--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 10 SEPT 2007																			--
-- Project name: ICRR firmware																--
--	Module name: MESS	  																			--
--	Description : 																					--
-- 	Master Event Synchronous Sequencer (MESS) module								--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MESS is
   port ( 
			 xSLWR			: in   std_logic;
			 xSTART		 	: in   std_logic;
			 xDONE		 	: in   std_logic;
			 xCLR_ALL	 	: in   std_logic;
			 xTEMP			: in   std_logic_vector (15 downto 0);
			 xSLAVE_DATA	: in   std_logic_vector (15 downto 0);
			 xFPGA_DATA    : out  std_logic_vector (15 downto 0);
			 xRSEL		   : out  std_logic_vector (3 downto 0);
			 xRSTRB    		: out  std_logic;
			 xRADDR			: out  std_logic_vector (15 downto 0));
end MESS;

architecture Behavioral of MESS is
	type STATE_TYPE is ( HDR_START,HDR_END,GND_STATE,
								SLAVE0,SLAVE1,SLAVE2,SLAVE3,
								TEMP0,TEMP1,TEMP2,TEMP3);
	signal STATE 			: STATE_TYPE;
	signal RADDR			: std_logic_vector(15 downto 0);
	signal PHASE_CNT		: std_logic_vector(15 downto 0);
	signal FPGA_DATA		: std_logic_vector(15 downto 0);
	signal RSTRB			: std_logic;
	signal EN_RSTRB		: std_logic;
	signal RSEL				: std_logic_vector(3 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF16
      port ( I  : in    std_logic_vector(15 downto 0); 
             O  : out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component BUF4
      port ( I  : in    std_logic_vector(3 downto 0); 
             O  : out   std_logic_vector(3 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	RSTRB <=  xSLWR when( EN_RSTRB = '1') else '0';	
--------------------------------------------------------------------------------
	process(xSLWR,xSTART,xDONE,xCLR_ALL)
	begin
		if xCLR_ALL = '1' or xDONE = '1' then
			RADDR 		<= (others=>'0');
			FPGA_DATA 	<= (others=>'0');
			RSEL		 	<= (others=>'0');
			PHASE_CNT 	<= (others=>'0');
			EN_RSTRB		<= '0';
			STATE 		<= HDR_START;
		elsif falling_edge(xSLWR) and xSTART = '1' then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when HDR_START =>			
					FPGA_DATA <= x"1234";
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0001";
					RADDR 	 <= (others=>'0');
					STATE 	 <= SLAVE0;				
--------------------------------------------------------------------------------					
				when SLAVE0 =>			
					FPGA_DATA <= xSLAVE_DATA;
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						STATE <= TEMP0;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP0 =>			
					FPGA_DATA <= xTEMP;
					RSEL 		 <= "0010";
					RADDR 	 <= RADDR + 1;
					STATE 	 <= SLAVE1;	
--------------------------------------------------------------------------------	
				when SLAVE1 =>			
					FPGA_DATA <= xSLAVE_DATA;
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						STATE <= TEMP1;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP1 =>			
					FPGA_DATA <= xTEMP;
					RSEL 		 <= "0100";
					RADDR 	 <= RADDR + 1;
					STATE 	 <= SLAVE2;	
--------------------------------------------------------------------------------	
				when SLAVE2 =>			
					FPGA_DATA <= xSLAVE_DATA;
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						STATE <= TEMP2;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP2 =>			
					FPGA_DATA <= xTEMP;
					RSEL 		 <= "1000";
					RADDR 	 <= RADDR + 1;
					STATE 	 <= SLAVE3;	
--------------------------------------------------------------------------------	
				when SLAVE3 =>			
					FPGA_DATA <= xSLAVE_DATA;
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						STATE <= TEMP3;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP3 =>			
					FPGA_DATA <= xTEMP;
					RSEL 		 <= "0000";
					EN_RSTRB	 <= '0';
					RADDR 	 <= (others=>'0');
					STATE 	 <= HDR_END;				
--------------------------------------------------------------------------------	
				when HDR_END =>			
					FPGA_DATA <= x"4321";
					STATE <= GND_STATE;	
--------------------------------------------------------------------------------	
				when GND_STATE =>			
					FPGA_DATA <= (others=>'0');
--------------------------------------------------------------------------------	
				when others =>	STATE<=HDR_START;																
			end case;
		end if;
	end process;		
--------------------------------------------------------------------------------							
	xBUF_OUT : BUF16 
	port map (
		I	=> FPGA_DATA,
		O	=> xFPGA_DATA);
--------------------------------------------------------------------------------							
	xBUF_RADDR : BUF16 
	port map (
		I	=> RADDR,
		O	=> xRADDR);
--------------------------------------------------------------------------------							
	xBUF_RSEL : BUF4 
	port map (
		I	=> RSEL,
		O	=> xRSEL);
--------------------------------------------------------------------------------							
	xBUF_RSTRB : BUF 
	port map (
		I	=> RSTRB,
		O	=> xRSTRB);
--------------------------------------------------------------------------------		
end Behavioral;