--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------																															
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 10 March 2007																			--
-- Project name: FDIRC_CAMAC firmware														--
-- FPGA chip :	Xilinx's SPARTAN3	xc3s200-208											   --
-- USB chip : CYPRESS CY7C68013  															--
--	Module name: PROGRESET        															--
--	Description : 																					--
-- 	progreset will be reset other module                        				--
--																										--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--------------------------------------------------------------------------------
--   								I/O Definitions		   						         --
--------------------------------------------------------------------------------

entity PROGRESET is
    Port ( 	xCLK     : 	in std_logic; 		-- CLOCK	48MHz
          	xWAKEUP  : 	in std_logic; 		-- Active High Powered up USB
				xCLR_ALL : 	out std_logic; 	-- Active High Clr_all
           	xGLRST   : 	out std_logic); 	-- RESET low-active
end PROGRESET;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

architecture Behavioral of PROGRESET is
	type State_type is(RESETD, NORMAL);
	signal state: State_type;	
	signal Clr_all	 	: std_logic;
	signal GLRST	 	: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	begin xRESET : process(xWAKEUP, xCLK) 
		variable i: integer range 0 to 144000000 :=0;
	begin	
		if xWAKEUP = '0' then -- asynchronous reset
			GLRST 	<= '0';
			Clr_all 	<= '1';		 				
			state <= RESETD;		
		elsif rising_edge(xCLK) then		
			case	state is
--------------------------------------------------------------------------------				
				when RESETD =>
					GLRST 	<= '0';
					Clr_all 	<= '1';
						if i = 144000000 then
							i:=0;
							state <= NORMAL;
						else
							if xWAKEUP = '1' then
								i := i + 1;
								state <= RESETD;
							else
								state <= RESETD;
							end if;
						end if;
--------------------------------------------------------------------------------
				when NORMAL =>
					if xWAKEUP = '1' then
						GLRST 	<= '1';
						Clr_all 	<= '0';
						state <=NORMAL;
					else
						state <= RESETD;
					end if;
--------------------------------------------------------------------------------
				when others =>
					state <= RESETD;
			end case;	
--------------------------------------------------------------------------------	
		end if;
	end process xRESET;
--------------------------------------------------------------------------------	
	xBUF_GLRST : BUF 
	port map (
		I  => GLRST,
		O  => xGLRST);			
--------------------------------------------------------------------------------
	xBUF_SCLK : BUF 
	port map (
		I  => Clr_all,
		O  => xCLR_ALL);			
--------------------------------------------------------------------------------
end Behavioral;