--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 26 JUNE 2007																			--
-- Project name: ICRR firmware																--
--	Module name: GND_16			  																--
--	Description : 																					--
-- 	GND_BUS 16 bits wide																		--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ROCKETIOv03 is
   port ( 
			 RXN   		: in    std_logic;
			 RXP   		: in    std_logic;
			 REFCLK  	: in    std_logic;
			 xRX_LOSS  	: in    std_logic;
			 xTX_DATA	: in    std_logic_vector (31 downto 0);
			 xROCLK		: out   std_logic;
			 xROCLKx2	: out   std_logic;
			 xRX_DATA	: out   std_logic_vector (31 downto 0);
			 xTX_BUSY	: out   std_logic;
			 xRX_BUSY	: out   std_logic;
			 TXN   		: out   std_logic;
          TXP   		: out   std_logic);
end ROCKETIOv03;

architecture Behavioral of ROCKETIOv03 is
	signal RXDATA		: std_logic_vector(31 downto 0);
	signal xxREFCLK	: std_logic;
	signal xREFCLK		: std_logic;
	signal USRCLK		: std_logic;
	signal USRCLK2		: std_logic;
	signal RX_USRCLK	: std_logic;
	signal TX_USRCLK	: std_logic;
	signal FO_RESET	: std_logic;
	signal DCM_LOCKED	: std_logic;
	signal TX_BUSY		: std_logic;
	signal RX_BUSY		: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component IBUFG
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component BUFG
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------
	COMPONENT mgt_top
	PORT(
		xTX_DATA		: IN std_logic_vector(31 downto 0);
		LOOPBACK_IN : IN std_logic_vector(1 downto 0);
		RXN_IN 		: IN std_logic;
		RXP_IN 		: IN std_logic;
		rxreset 		: IN std_logic;
		txreset 		: IN std_logic;
		refclk 		: IN std_logic;
		txusrclk 	: IN std_logic;
		txusrclk2 	: IN std_logic;
		rxusrclk 	: IN std_logic;
		rxusrclk2 	: IN std_logic;
		rxpolarity 	: IN std_logic;
		txpolarity 	: IN std_logic;          
		TXN_OUT 		: OUT std_logic;
		TXP_OUT 		: OUT std_logic;
		xTX_BUSY		: OUT std_logic;
		rx_data 		: OUT std_logic_vector(31 downto 0)
		);
	END COMPONENT;
--------------------------------------------------------------------------------
	COMPONENT clk_gen
	PORT(
		xREFCLK 		: IN std_logic;          
		USRCLK_M 	: OUT std_logic;
		USRCLK2_M 	: OUT std_logic;
		DCM_LOCKED 	: OUT std_logic
		);
	END COMPONENT;
--------------------------------------------------------------------------------
	COMPONENT reset_gen
	PORT(
		reset_in 	: IN std_logic;
		clk 			: IN std_logic;
		locked 		: IN std_logic;          
		reset_out 	: OUT std_logic
		);
	END COMPONENT;
--------------------------------------------------------------------------------	
	component BUF32
    Port ( 
			  I   : in   std_logic_vector (31 downto 0);
			  O	: out  std_logic_vector (31 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
	xIBUFG_REFCLK : IBUFG 
	port map (
		I  => REFCLK,
		O  => xxREFCLK);
--------------------------------------------------------------------------------	
	xBUFG_REFCLK : BUFG 
	port map (
		I  => xxREFCLK,
		O  => xREFCLK);
--------------------------------------------------------------------------------	
	INST_reset_gen : reset_gen 
	port map (
		reset_in		=> xRX_LOSS,
		clk 			=> USRCLK2,
		locked  		=> DCM_LOCKED,
		reset_out	=> FO_RESET);
--------------------------------------------------------------------------------	
	INST_clk_gen : clk_gen 
	port map (
		xREFCLK		=> xREFCLK,
		USRCLK_M 	=> USRCLK,
		USRCLK2_M  	=> USRCLK2,
		DCM_LOCKED	=> DCM_LOCKED);
--------------------------------------------------------------------------------	
	RX_USRCLK <= not USRCLK;
	TX_USRCLK <= not USRCLK;
--------------------------------------------------------------------------------	
	INST_MGT_TOP : mgt_top 
	port map (
		xTX_BUSY    => TX_BUSY,
		xTX_DATA		=> xTX_DATA,
		LOOPBACK_IN	=> "00",
		RXN_IN 		=> RXN,
		RXP_IN  		=> RXP,
		rxreset  	=> FO_RESET,
		txreset  	=> FO_RESET,
		refclk  		=> xREFCLK,
		txusrclk  	=> TX_USRCLK,
		txusrclk2  	=> USRCLK2,
		rxusrclk  	=> RX_USRCLK,
		rxusrclk2  	=> USRCLK2,
		rxpolarity  => '0',
		txpolarity  => '0',
		TXN_OUT  	=> TXN,
		TXP_OUT  	=> TXP,		
		rx_data  	=> RXDATA);
--------------------------------------------------------------------------------	
	xBUF_TX_BUSY : BUF 
	port map (
		I  => TX_BUSY,
		O  => xTX_BUSY);
--------------------------------------------------------------------------------	
	xBUF_RX_BUSY : BUF 
	port map (
		I  => RX_BUSY,
		O  => xRX_BUSY);
--------------------------------------------------------------------------------	
	xBUF_USRCLK2 : BUF 
	port map (
		I  => USRCLK2,
		O  => xROCLK);
--------------------------------------------------------------------------------	
	xBUF_USRCLK : BUF 
	port map (
		I  => USRCLK,
		O  => xROCLKx2);
--------------------------------------------------------------------------------	
	xBUF_RXDATA : BUF32 
	port map (
		I  => RXDATA,
		O  => xRX_DATA);
--------------------------------------------------------------------------------	
	process(RXDATA)
	begin
		if RXDATA = x"BC95B5B5" then
			RX_BUSY <= '1';
		else 
			RX_BUSY <= '0';
		end if;
	end process;
--------------------------------------------------------------------------------
end Behavioral;