--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 29 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: TRIG_DAC		  																--
--	Description : 																					--
-- 	Top Level for Trigger Dac Control													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity R_WR_CTRL is
   port ( xCLK		     	: in  std_logic;
			 xSTART			: in  std_logic;
			 xSLAVE_WBUSY 	: in  std_logic;
			 xWSTRB		 	: in  std_logic;
			 xRSTRB		 	: in  std_logic;
			 xWSEL    		: in  std_logic_vector (3 downto 0);
			 xRSEL    		: in  std_logic_vector (3 downto 0);
          xCLR_ALL  		: in  std_logic;
			 xDSTRB			: out std_logic;
			 xMAIN_RBUSY	: out std_logic;
          xSEL    		: out std_logic_vector (3 downto 0);
			 xR_WR	   	: out std_logic);
end R_WR_CTRL;

architecture Behavioral of R_WR_CTRL is
	signal SEL				: std_logic_vector(3 downto 0);
	signal DSTRB 			: std_logic;
	signal MAIN_RBUSY		: std_logic;
	signal R_WR				: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF4
    Port ( 
			  I   : in   std_logic_vector (3 downto 0);
			  O	: out  std_logic_vector (3 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	xBUF_SEL : BUF4 
	port map (
		I	=> SEL,
		O	=>	xSEL);
--------------------------------------------------------------------------------
	xBUF_DSTRB : BUF 
	port map (
		I	=> DSTRB,
		O	=>	xDSTRB);
--------------------------------------------------------------------------------
	xBUF_MAIN_RBUSY : BUF 
	port map (
		I	=> MAIN_RBUSY,
		O	=>	xMAIN_RBUSY);
--------------------------------------------------------------------------------
	xBUF_R_WR : BUF 
	port map (
		I	=> R_WR,
		O	=>	xR_WR);
--------------------------------------------------------------------------------
	process(xCLK,xSTART,xSLAVE_WBUSY,xCLR_ALL)
	begin
		if xCLR_ALL = '1' then
			MAIN_RBUSY 	<= '0';
		elsif falling_edge(xCLK) then
			if xSTART = '1' and xSLAVE_WBUSY = '0' then
				MAIN_RBUSY 	<= '1';
			else
				MAIN_RBUSY 	<= '0';
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------	
	process(MAIN_RBUSY,xSLAVE_WBUSY)
	begin
		if MAIN_RBUSY = '1' then
			SEL 	<= xRSEL;
			DSTRB <= xRSTRB;
			R_WR 	<= '0';
		elsif xSLAVE_WBUSY = '1' then
			SEL 	<= xWSEL;
			DSTRB <= xWSTRB;
			R_WR 	<= '1';
		else
			SEL 	<= (others=>'0');
			DSTRB <= '0';
			R_WR 	<= '0';
		end if;
	end process;
--------------------------------------------------------------------------------
end Behavioral;