--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 25 JUNE 2007																			--
-- Project name: ICRR firmware																--
--	Module name: TEMP_TOP		  																--
--	Description : 																					--
-- 	Temperature monitoring module															--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TEMP_TOP is
   port ( 
			 TDIN		      : in    std_logic_vector(3 downto 0);	
			 xCLK_15MHz	 	: in    std_logic;
			 xRADDR	      : in    std_logic_vector(1 downto 0);	
			 xCLR_ALL		: in    std_logic;
			 T_CS				: out   std_logic;
			 T_SCLK			: out   std_logic;			 
			 --DEBUG_T_CS		: out   std_logic;
			 --DEBUG_T_SCLK	: out   std_logic;
			 --DEBUG_T_DOUT	: out   std_logic;
			 xTEMP			: out   std_logic_vector (15 downto 0));
end TEMP_TOP;

architecture Behavioral of TEMP_TOP is
	type STATE_TYPE is ( ENABLE, INTIALIZE, DIGITIZE, 
								LOAD, WAIT_0, WAIT_1);   
	signal STATE 		   : STATE_TYPE;
	signal TEMP				: std_logic_vector(15 downto 0);
	signal TEMP0			: std_logic_vector(15 downto 0);
	signal TEMP1			: std_logic_vector(15 downto 0);
	signal TEMP2			: std_logic_vector(15 downto 0);
	signal TEMP3			: std_logic_vector(15 downto 0);
	signal xCNT				: std_logic_vector(4 downto 0);	
	signal xCNT_WAIT		: std_logic_vector(16 downto 0);	
	signal xCNT_DIG		: std_logic_vector(4 downto 0);
	signal xCNT_A			: std_logic_vector(4 downto 0);	
	signal xT_SCLK			: std_logic;
	signal EN_SCLK			: std_logic;	
	signal xT_CS			: std_logic;	
	signal EN_STRB			: std_logic;
	signal xTDIN			: std_logic_vector(3 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component OBUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component BUF16
      port ( I  : in    std_logic_vector(15 downto 0); 
             O  : out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------
   component IBUF4X
      port ( I  : in    std_logic_vector(3 downto 0); 
             O  : out   std_logic_vector(3 downto 0));
   end component;
--------------------------------------------------------------------------------
	component NAND2
   port ( 	 I0 : in    std_logic; 
             I1 : in    std_logic; 
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------	
	xIBUF_TDIN : IBUF4X 
	port map (
		I  => TDIN,
		O  => xTDIN);			
--------------------------------------------------------------------------------
	process(xCLR_ALL,xCLK_15MHz)
	begin
		if xCLR_ALL = '1' then
			xCNT 			<= (others=>'0');
			xCNT_DIG		<= (others=>'0');
			xCNT_WAIT	<= (others=>'0');
			EN_SCLK 		<= '0';
			xT_CS 		<= '1';
			EN_STRB 		<= '0';
			STATE       <= ENABLE;
		elsif rising_edge(xCLK_15MHz) then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when ENABLE =>			
					xT_CS <= '0';
					STATE <= INTIALIZE;
--------------------------------------------------------------------------------	
				when INTIALIZE =>			
					EN_SCLK <= '1';
					STATE <= DIGITIZE;
--------------------------------------------------------------------------------	
				when DIGITIZE =>			
					xCNT_DIG <= xCNT_DIG + 1;
					if xCNT_DIG = 15 then
						xCNT_DIG	<= (others=>'0');
						xT_CS <= '1';
						EN_SCLK <= '0';
						STATE <= LOAD;
					end if;
--------------------------------------------------------------------------------	
				when LOAD =>			
					xCNT_A <= xCNT_A + 1;
					xCNT <= xCNT_A - 1;
					EN_STRB <= '1';
					if xCNT_A = 16 then
						xCNT_A	<= (others=>'0');
						STATE <= WAIT_0;
					end if;
--------------------------------------------------------------------------------	
				when WAIT_0 =>			
					xT_CS <= '1';
					EN_SCLK <= '0';
					EN_STRB <= '0';
					xCNT	<= (others=>'0');
					STATE <= WAIT_1;		
--------------------------------------------------------------------------------	
				when WAIT_1 =>			
					xT_CS <= '1';
					EN_SCLK <= '0';
					EN_STRB <= '0';
					xCNT_WAIT <= xCNT_WAIT + 1;
					if xCNT_WAIT = 100000 then
						xCNT_WAIT	<= (others=>'0');
						STATE <= ENABLE;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE<=ENABLE;																
			end case;
		end if;
	end process;	
--------------------------------------------------------------------------------
	process(xCNT_DIG,xCLR_ALL,xT_SCLK,xT_CS)
	begin
		if xCLR_ALL = '1' then
			TEMP0(15 downto 0) <= (others=>'0');
			TEMP1(15 downto 0) <= (others=>'0');
			TEMP2(15 downto 0) <= (others=>'0');
			TEMP3(15 downto 0) <= (others=>'0');
		elsif xT_CS = '0' then
			if xCNT_DIG = 0 and rising_edge(xT_SCLK) then
				TEMP0(15) <= '0';
				TEMP1(15) <= '0';
				TEMP2(15) <= '0';
				TEMP3(15) <= '0';
			end if;
			if xCNT_DIG = 1 and rising_edge(xT_SCLK) then
				TEMP0(14) <= '0';
				TEMP1(14) <= '0';
				TEMP2(14) <= '0';
				TEMP3(14) <= '0';
			end if;
			if xCNT_DIG = 2 and rising_edge(xT_SCLK) then
				TEMP0(13) <= xTDIN(0);
				TEMP1(13) <= xTDIN(1);
				TEMP2(13) <= xTDIN(2);
				TEMP3(13) <= xTDIN(3);
			end if;
			if xCNT_DIG = 3 and rising_edge(xT_SCLK) then
				TEMP0(12) <= xTDIN(0);
				TEMP1(12) <= xTDIN(1);
				TEMP2(12) <= xTDIN(2);
				TEMP3(12) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 4 and rising_edge(xT_SCLK) then
				TEMP0(11) <= xTDIN(0);
				TEMP1(11) <= xTDIN(1);
				TEMP2(11) <= xTDIN(2);
				TEMP3(11) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 5 and rising_edge(xT_SCLK) then
				TEMP0(10) <= xTDIN(0);
				TEMP1(10) <= xTDIN(1);
				TEMP2(10) <= xTDIN(2);
				TEMP3(10) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 6 and rising_edge(xT_SCLK) then
				TEMP0(9) <= xTDIN(0);
				TEMP1(9) <= xTDIN(1);
				TEMP2(9) <= xTDIN(2);
				TEMP3(9) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 7 and rising_edge(xT_SCLK) then
				TEMP0(8) <= xTDIN(0);
				TEMP1(8) <= xTDIN(1);
				TEMP2(8) <= xTDIN(2);
				TEMP3(8) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 8 and rising_edge(xT_SCLK) then
				TEMP0(7) <= xTDIN(0);
				TEMP1(7) <= xTDIN(1);
				TEMP2(7) <= xTDIN(2);
				TEMP3(7) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 9 and rising_edge(xT_SCLK) then
				TEMP0(6) <= xTDIN(0);
				TEMP1(6) <= xTDIN(1);
				TEMP2(6) <= xTDIN(2);
				TEMP3(6) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 10 and rising_edge(xT_SCLK) then
				TEMP0(5) <= xTDIN(0);
				TEMP1(5) <= xTDIN(1);
				TEMP2(5) <= xTDIN(2);
				TEMP3(5) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 11 and rising_edge(xT_SCLK) then
				TEMP0(4) <= xTDIN(0);
				TEMP1(4) <= xTDIN(1);
				TEMP2(4) <= xTDIN(2);
				TEMP3(4) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 12 and rising_edge(xT_SCLK) then
				TEMP0(3) <= xTDIN(0);
				TEMP1(3) <= xTDIN(1);
				TEMP2(3) <= xTDIN(2);
				TEMP3(3) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 13 and rising_edge(xT_SCLK) then
				TEMP0(2) <= xTDIN(0);
				TEMP1(2) <= xTDIN(1);
				TEMP2(2) <= xTDIN(2);
				TEMP3(2) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 14 and rising_edge(xT_SCLK) then
				TEMP0(1) <= xTDIN(0);
				TEMP1(1) <= xTDIN(1);
				TEMP2(1) <= xTDIN(2);
				TEMP3(1) <= xTDIN(3);			
			end if;
			if xCNT_DIG = 15 and rising_edge(xT_SCLK) then
				TEMP0(0) <= xTDIN(0);
				TEMP1(0) <= xTDIN(1);
				TEMP2(0) <= xTDIN(2);
				TEMP3(0) <= xTDIN(3);			
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------
	process(xRADDR)
	begin
		if xRADDR = 0 then
			TEMP <= TEMP0;
		elsif xRADDR = 1 then
			TEMP <= TEMP1;
		elsif xRADDR = 2 then
			TEMP <= TEMP2;
		elsif xRADDR = 3 then
			TEMP <= TEMP3;
		end if;
	end process;
--------------------------------------------------------------------------------
	xBUF_TEMP : BUF16 
	port map (
		I(15 downto 0)=> TEMP(15 downto 0),
		O(15 downto 0)=> xTEMP(15 downto 0));
--------------------------------------------------------------------------------
	xNAND2_SCLK : NAND2 
	port map (
		I0  => xCLK_15MHz,
		I1  => EN_SCLK,
		O   => xT_SCLK);
--------------------------------------------------------------------------------
	xOBUF_T_CS : OBUF 
	port map (
		I  => xT_CS,
		O  => T_CS);		
--------------------------------------------------------------------------------
	xOBUF_T_SCLK : OBUF 
	port map (
		I  => xT_SCLK,
		O  => T_SCLK);	
--------------------------------------------------------------------------------
--	xDEBUG_T_CS: BUF 
--	port map (
--		I  => xT_CS,
--		O  => DEBUG_T_CS);
--------------------------------------------------------------------------------
--	xDEBUG_T_SCLK: BUF 
--	port map (
--		I  => xT_SCLK,
--		O  => DEBUG_T_SCLK);
--------------------------------------------------------------------------------
--	xDEBUG_T_DOUT: BUF 
--	port map (
--		I  => xT_DOUT,
--		O  => DEBUG_T_DOUT);		
--------------------------------------------------------------------------------
end Behavioral;