--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 16 JUNE 2007																			--
-- Project name: ICRR firmware																--
--	Module name: USBread  																		--
--	Description : 																					--
-- 	USB 2.0 reading data from PC module													--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TX_DATA is
   port ( 
			 xROCLK	  		: in   std_logic;
			 xROCLKx2		: in   std_logic;
			 xTX_BUSY		: in   std_logic;
			 xSTART			: in   std_logic;
			 xTEMP			: in   std_logic_vector (15 downto 0);
			 xSLAVE_DATA	: in   std_logic_vector (15 downto 0);	
			 xDONE			: in   std_logic;
			 xCLR_ALL		: in   std_logic;
			 xRSEL		   : out  std_logic_vector (3 downto 0);
			 xRSTRB    		: out  std_logic;
			 xRADDR			: out  std_logic_vector (15 downto 0);
			 xTX_DATA		: out  std_logic_vector (31 downto 0));
end TX_DATA;

architecture Behavioral of TX_DATA is
	type STATE_TYPE is ( IDLE,HDR_START,HDR_END,GND_STATE,
								SLAVE0,SLAVE1,SLAVE2,SLAVE3,
								TEMP0,TEMP1,TEMP2,TEMP3);
	signal STATE 			: STATE_TYPE;
	signal TX_DATA 		: std_logic_vector(31 downto 0);
	signal RADDR			: std_logic_vector(15 downto 0);
	signal PHASE_CNT		: std_logic_vector(15 downto 0);
	signal RSTRB			: std_logic;
	signal EN_RSTRB		: std_logic;
	signal RSEL				: std_logic_vector(3 downto 0);
	signal ROCLK_90		: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------
   component BUF4
      port ( I  : in    std_logic_vector(3 downto 0); 
             O  : out   std_logic_vector(3 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF32
    Port ( 
			  I   : in   std_logic_vector (31 downto 0);
			  O	: out  std_logic_vector (31 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	process(xROCLKx2)
	begin
		if falling_edge(xROCLKx2) then
			ROCLK_90 <= xROCLK;
		end if;
	end process;
--------------------------------------------------------------------------------
	RSTRB <=  EN_RSTRB and ROCLK_90 and not(xTX_BUSY);	
--------------------------------------------------------------------------------
	process(xCLR_ALL,xROCLK,xTX_BUSY,xDONE)
	begin
		if xCLR_ALL = '1' or xDONE = '1' then
			TX_DATA 		<= (others=>'0');
			RADDR 		<= (others=>'0');
			RSEL		 	<= (others=>'0');
			PHASE_CNT 	<= (others=>'0');
			EN_RSTRB		<= '0';
			STATE 		<= IDLE;
		elsif falling_edge(xROCLK) and xTX_BUSY = '0' then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when IDLE =>			
					TX_DATA 		<= (others=>'0');
					if xSTART = '1' then
						EN_RSTRB	<= '0';
						RSEL 		 <= "0001";
						STATE 	<= HDR_START;
					else
					   EN_RSTRB	<= '0';
						RSEL 		 <= "0000";
						STATE 	<= IDLE;
					end if;
--------------------------------------------------------------------------------						
				when HDR_START =>			
					TX_DATA	<= x"12341234";
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0001";
					RADDR 	 <= (others=>'0');
					PHASE_CNT <= (others=>'0');
					STATE 	 <= SLAVE0;		
--------------------------------------------------------------------------------					
				when SLAVE0 =>			
					TX_DATA <= x"0000" & xSLAVE_DATA;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0001";
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						RSEL 		 <= "0010";
						STATE <= TEMP0;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP0 =>			
					TX_DATA <= x"0000" & xTEMP;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0010";
					RADDR 	 <= RADDR + 1;
					STATE 	 <= SLAVE1;	
--------------------------------------------------------------------------------	
				when SLAVE1 =>			
					TX_DATA <= x"0000" & xSLAVE_DATA;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0010";
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						RSEL 		 <= "0100";
						STATE <= TEMP1;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP1 =>			
					TX_DATA <= x"0000" & xTEMP;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0100";
					RADDR 	 <= RADDR + 1;
					STATE 	 <= SLAVE2;	
--------------------------------------------------------------------------------	
				when SLAVE2 =>			
					TX_DATA <= x"0000" & xSLAVE_DATA;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "0100";
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						RSEL 		 <= "1000";
						STATE <= TEMP2;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP2 =>			
					TX_DATA <= x"0000" & xTEMP;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "1000";
					RADDR 	 <= RADDR + 1;
					STATE 	 <= SLAVE3;	
--------------------------------------------------------------------------------	
				when SLAVE3 =>			
					TX_DATA <= x"0000" & xSLAVE_DATA;
					EN_RSTRB	 <= '1';
					RSEL 		 <= "1000";
					PHASE_CNT <= PHASE_CNT + 1;
					if PHASE_CNT = 8199 then --8199
						PHASE_CNT <= (others=>'0');
						RSEL 		 <= "0000";
						STATE <= TEMP3;	
					end if;
--------------------------------------------------------------------------------					
				when TEMP3 =>			
					TX_DATA 	 <= x"0000" & xTEMP;
					EN_RSTRB	 <= '0';
					RSEL 		 <= "0000";
					RADDR 	 <= (others=>'0');
					STATE 	 <= HDR_END;		
--------------------------------------------------------------------------------	
				when HDR_END =>			
					TX_DATA <= x"43214321";
					STATE <= GND_STATE;	
--------------------------------------------------------------------------------	
				when GND_STATE =>			
					TX_DATA <= (others=>'0');					
--------------------------------------------------------------------------------						
				when others =>	STATE<=IDLE;																
			end case;
		end if;
	end process;
--------------------------------------------------------------------------------
	xBUF_RSEL : BUF4 
	port map (
		I	=> RSEL,
		O	=> xRSEL);
--------------------------------------------------------------------------------							
	xBUF_RSTRB : BUF 
	port map (
		I	=> RSTRB,
		O	=> xRSTRB);
--------------------------------------------------------------------------------		
	xBUF_RADDR : BUF16 
	port map (
		I	=> RADDR,
		O	=> xRADDR);
--------------------------------------------------------------------------------
	xBUF_TX_DATA : BUF32 
	port map (
		I	=> TX_DATA,
		O	=> xTX_DATA);
--------------------------------------------------------------------------------
end Behavioral;