`timescale 1ns / 1ps
module rx_check_ctrl(
							aligned_data
							,aligned_rxisk3
							,sync
							,rxusrclk2
							,reset
							,rx_data
							,trans_flag
							);

			input	[31:0] aligned_data;
			input	       aligned_rxisk3;
			input        sync;
			input			 rxusrclk2;
			input        reset;

			output[31:0] rx_data;
			output       trans_flag;

wire       enable;
wire [31:0]last_data;
reg  [31:0]act_data;
reg        sync_reg;
reg  [15:0]err_cnt;
reg        err;

//synthesis attribute KEEP of err_cnt is "true"

reg        trans_flag;

parameter ERR_MAX = 256;

assign rx_data = aligned_data;

assign rst_stat = (aligned_data==32'h7C95B5B5 && aligned_rxisk3) || reset;

always@(posedge rxusrclk2)
begin
	if(rst_stat)
		trans_flag <= 1'b0;
	else if(aligned_data==32'hFC95B5B5 && aligned_rxisk3)
		trans_flag <= 1'b1;
	else
		trans_flag <= trans_flag;
end

assign enable = sync && !aligned_rxisk3;


assign last_data = aligned_data;

always@(posedge rxusrclk2)
begin
	if(rst_stat)
		begin
			act_data <= 32'b0;
		end
	else if(enable)
		act_data <= last_data + 1;
	else
		act_data <= act_data;
end

always@(posedge rxusrclk2)
begin
	if(rst_stat)
	begin
		err_cnt  <= 16'b0;
		sync_reg <= 1'b0;
	end
	else if(enable)
	begin
		sync_reg <= sync;
		if(sync)
		begin
			if(sync_reg)
			begin
				if(enable && err)
				begin
					if(err_cnt < ERR_MAX)
						err_cnt <= err_cnt + 1;
					else
						err_cnt <= err_cnt;
				end
			else
				err_cnt <= err_cnt;
			end
		end
	else
		err_cnt  <= err_cnt;
	end
end

always@(negedge rxusrclk2)
begin
	if(rst_stat)
		begin
			err <= 1'b0;
		end
	else if(enable)
		begin
			if(last_data != act_data)
				err <= 1'b1;
			else
				err <=1'b0;
		end
	else
		begin
			err <= 1'b0;
		end
end

endmodule
